import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JPanel;


/**
 * A class representing a deck of cards drawn splayed on a panel
 * 
 * @author Joel Ross
 * @version Sp14
 */
public class DrawnDeck extends JPanel
{
	//constants for convenience
	public static final int CARD_SPACING = 13; //space between each card
	public static final int OFFSET_W = 20; //offset on the left/right
	public static final int OFFSET_H = 50; //offset on the top/bottom
	public static final int SIZE = Card.NUM_SUITS * Card.NUM_RANKS; //number of cards in the deck
	
	public static final int PANEL_WIDTH = CARD_SPACING * SIZE + (DrawableCard.WIDTH - CARD_SPACING) + (OFFSET_W*2); //width of panel
	public static final int PANEL_HEIGHT = OFFSET_H*2 + DrawableCard.HEIGHT; //height of panel
	
	private DrawableCard[] deck;
	private int selectedIndex;

	/**
	 * Constructs a new standard deck of drawable cards
	 */
	public DrawnDeck()
	{
		this.setPreferredSize(new Dimension(PANEL_WIDTH,PANEL_HEIGHT));
		this.setBackground(new Color(43,105,42)); //nice green for background
		
		BufferedImage[][] faces = loadImageTiles("assets/cards.png", Card.NUM_RANKS, Card.NUM_SUITS);

		deck = new DrawableCard[SIZE];
		int count = 0;
		for(int i=1; i<=Card.NUM_SUITS; i++)
		{
			for(int j=1; j<=Card.NUM_RANKS; j++)
			{
				deck[count] = new DrawableCard(j, i, faces[j-1][i-1]);
				count++;
			}
		}
		selectedIndex = 0;
	}

	/**
	 * Selects a card from the deck
	 * @param index the index of the card to select
	 */
	public void select(int index)
	{
		selectedIndex = index;
	}
	
	/**
	 * Flips a card in the deck
	 * @param index the index of the card to flip
	 */
	public void flipCard(int index)
	{
		deck[index].flip();
		this.repaint();
	}
	
	/**
	 * Resets the deck so that all the cards are face up
	 */
	public void reset()
	{
		for(DrawableCard card : deck)
		{
			if(!card.isFaceUp())
				card.flip();
		}
		repaint();
	}
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D)g;
		
		for(int i=0; i<deck.length; i++) //draw each card in the deck
		{
			deck[i].draw(g2d, OFFSET_W+(i*CARD_SPACING), OFFSET_H);
		}

		//draw the selected card, on top of the others
		DrawableCard selected = deck[selectedIndex];
		selected.draw(g2d, OFFSET_W+(selectedIndex*CARD_SPACING), OFFSET_H);
		Stroke oldStroke = g2d.getStroke(); //store old settings
		g2d.setStroke(new BasicStroke(2));
		Paint oldPaint = g2d.getPaint();
		g2d.setPaint(Color.YELLOW);
		g2d.drawRect(OFFSET_W+(selectedIndex*CARD_SPACING), OFFSET_H, DrawableCard.WIDTH, DrawableCard.HEIGHT);
		g2d.setStroke(oldStroke); //reset old settings
		g2d.setPaint(oldPaint);
	
	}
	
	/**
	 * Loads an image that provides tiles for drawing the deck of cards, and returns an array of those images
	 * Code adapted from Adam Smith.
	 * 
	 * @param filename The image file containing the tile
	 * @param numTilesWide The number of tiles wide the image is
	 * @param numTilesHigh The number of tiles high the image is
	 * @return A 2D array of BufferedImages representing the deck of cards. Array can be accessed as [rank][suit]
	 */
	public BufferedImage[][] loadImageTiles(String filename, int numTilesWide, int numTilesHigh)
	{
		BufferedImage master;
		
		try {
		    master = ImageIO.read(new File(filename));
		}
		catch (IOException e) {
		    System.err.println("Error loading " +filename+ ":");
		    e.printStackTrace();
		    return null;
		}
		
		BufferedImage[][] tiles = new BufferedImage[numTilesWide][numTilesHigh]; //initialize array
		int tileWidth = master.getWidth() / numTilesWide;
		int tileHeight = master.getHeight() / numTilesHigh;

		for (int i=0; i<tiles.length; i++)
		{
		    for (int j=0; j<tiles[i].length; j++)
		    {
		    	tiles[i][j] = master.getSubimage(i*tileWidth, j*tileHeight, tileWidth, tileHeight); //slice image
		    }
		}

		return tiles;
	}
}
